#===============================================================================
# CERN (BE-CO-HT)
# Register test for CONV-TTL-BLO
#===============================================================================
# author: Theodor Stana (t.stana@cern.ch)
#
# date of creation: 2013-08-13
#
# version: 1.0
#
# description: 
#
# dependencies:
#
# references:
#===============================================================================
# GNU LESSER GENERAL PUBLIC LICENSE
#===============================================================================
# This source file is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation; either version 2.1 of the License, or (at your
# option) any later version. This source is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details. You should have
# received a copy of the GNU Lesser General Public License along with this
# source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
#===============================================================================
# last changes:
#    2013-08-13    Theodor Stana     t.stana@cern.ch     File created
#===============================================================================
#  TODO: - 
#===============================================================================

import random
import sys
import time
sys.path.append("../vbcp")
from vbcp import *

def spi_transfer(cs, dat):
  retval = 0;
  ctb.write(0x90, (((cs << 10) | 0x100) | dat))
  while (retval & (1 << 9) == 0):
    retval = ctb.read(0x90)
  return retval & 0xFF

def flash_write(addr, dat):
  spi_transfer(0,0)
  spi_transfer(1,0x06)
  spi_transfer(0,0)
  spi_transfer(1,0x02)
  spi_transfer(1,(addr & 0xFF0000) >> 16)
  spi_transfer(1,(addr & 0xFF00) >> 8)
  spi_transfer(1,(addr & 0xFF))
  for i in range(len(dat)):
    spi_transfer(1,dat[i])
  spi_transfer(0,0)

def flash_read(addr, nrbytes):
  ret = []
  spi_transfer(0,0)
  spi_transfer(1,0x0b)
  spi_transfer(1,(addr & 0xFF0000) >> 16)
  spi_transfer(1,(addr & 0xFF00) >> 8)
  spi_transfer(1,(addr & 0xFF))
  spi_transfer(1,0)
  for i in range(nrbytes):
    ret.append(spi_transfer(1,0))
  spi_transfer(0,0)
  return ret

def flash_serase(addr):
  ret = []
  spi_transfer(0,0)
  spi_transfer(1,0x06)
  spi_transfer(0,0)
  spi_transfer(1,0xd8)
  spi_transfer(1,(addr & 0xFF0000) >> 16)
  spi_transfer(1,(addr & 0xFF00) >> 8)
  spi_transfer(1,(addr & 0xFF))
  spi_transfer(0,0)

def flash_berase():
  spi_transfer(0,0)
  spi_transfer(1,0x06)
  spi_transfer(0,0)
  spi_transfer(1,0xc7)
  spi_transfer(0,0)

def flash_rsr():
  spi_transfer(0,0)
  spi_transfer(1,0x05)
  ret = spi_transfer(1,0)
  spi_transfer(0,0)
  return ret




if __name__ == "__main__":
  ip   = "192.168.20.22"  # raw_input("Crate IP? ")
  user = "admin" # raw_input("Username? ")
  pwd  = "ADMIN" # raw_input("Password? ")

  # Wait for proper slot number
  while 1:
    try:
      slot = 4
      #slot = raw_input("Slot no.? ")
      #slot = int(slot)
      ctb  = VBCP(ip, user, pwd, slot)
      break
    except TypeError as e:
      print "Please input a decimal slot number."
    except SlotError as e:
      print e.strerror
    except KeyboardInterrupt:
      sys.exit();
    except:
      print "Unexpected error: ", sys.exc_info()[0]


  # Open VBCP connection
  ctb.open_conn()

  # Init counters
  #wexcep_cnt   = 0
  #rexcep_cnt   = 0
  #mismatch_cnt = 0

#  print "erasing"
#
#  flash_berase()
#  while (flash_rsr() & 0x01):
#    pass
#
#  print "done!"


  # read from flash
  print "BEFORE"
  tr0 = time.time()
  rd = flash_read(0x000000,256)
  tr1 = time.time()
  rd = "".join(["0x%02X " % b for b in rd])
  print rd

  tr2 = time.time()
  rd = flash_read(0x000100,256)
  tr3 = time.time()
  rd = "".join(["0x%02X " % b for b in rd])
  print rd

  print "ERASE"
  te1 = time.time()
  flash_serase(0x0010)
  while (flash_rsr() & 0x01):
    pass
  te2 = time.time()
  print "erase time: %2.6f" % (te2-te1)

  print "WRITE"
  f = open("conv.txt",'r')
  addr = 0
  tdat = []
  twr  = []
  twa  = []
  for fdata in f:
    print addr
    data = []
    try:
      t1 = time.time()
      for i in xrange(256):
        data.append(int(fdata[0:2],16))
        fdata = fdata[2:]
      t2 = time.time()
    except ValueError:
      print "reached end"
      break
    finally:
      t3 = time.time()
      flash_write(addr, data)
      t4 = time.time()
      while (flash_rsr() & 0x01):
        pass
      t5 = time.time()
      addr += 256
      if (addr == 256**2):
        break;
      tdat.append(t2-t1)
      twr.append(t4-t3)
      twa.append(t5-t4)

      #print data
      #print len(data)

  # Close file handle
  f.close()

  print "DONE!"
  print "read1 time: %2.6f" % float(tr1-tr0)
  print "read2 time: %2.6f" % float(tr3-tr2)
  print "data time: %2.6f" % float(sum(tdat)/len(tdat))
  print "write time: %2.6f" % float(sum(twr)/len(twr)) #(t4-t3)
  print "wait time: %2.6f" % float(sum(twa)/len(twa)) #(t5-t4)


  # Close VBCP connection and print final values
  ctb.close_conn()

